﻿// Copyright 1998-2019 Epic Games, Inc. All Rights Reserved.

#include "StoveSDKPlugin.h"
#include "Core.h"
#include "Modules/ModuleManager.h"
#include "Interfaces/IPluginManager.h"
#include "GenericPlatform/GenericPlatformMisc.h"
#include "../../../Binaries/Thirdparty/StoveSDKPluginLibrary/include/StovePCSDK.h"


#define LOCTEXT_NAMESPACE "FStoveSDKPluginModule"

void FStoveSDKPluginModule::StartupModule()
{
    // This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module

    // Get the base directory of this plugin
    FString BaseDir = IPluginManager::Get().FindPlugin("StoveSDKPlugin")->GetBaseDir();
    FString LibraryPath;
    FString LibraryName = "StovePCSDK.dll";
    // Add on the relative location of the third party dll and load it
#ifdef _WIN64
    LibraryPath = FPaths::Combine(*BaseDir, TEXT("Binaries/ThirdParty/StoveSDKPluginLibrary/bin/x64/Release/"));
#else
    LibraryPath = FPaths::Combine(*BaseDir, TEXT("Binaries/ThirdParty/StoveSDKPluginLibrary/bin/Win32/Release/"));
#endif

    FPlatformProcess::PushDllDirectory(*LibraryPath);
    _StoveSDKLibraryHandle = !LibraryPath.IsEmpty() ? FPlatformProcess::GetDllHandle(*(LibraryPath + LibraryName)) : nullptr;
}


void FStoveSDKPluginModule::ShutdownModule()
{
    // This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
    // we call this function before unloading the module.

    // Free the dll handle
    FPlatformProcess::FreeDllHandle(_StoveSDKLibraryHandle);
    _StoveSDKLibraryHandle = nullptr;
}

StovePCResult FStoveSDKPluginModule::Init(const StovePCConfig& config, const StovePCCallback& callback)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : StovePC_Init -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_Init(config, callback);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_Init is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_Init is result = %d"), result);
        }
        return result; 
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
    
}

StovePCResult FStoveSDKPluginModule::IAPInit(const char* shopKey)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : StovePC_IAPInit -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_IAPInit(shopKey);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_IAPInit is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_IAPInit is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCInitState FStoveSDKPluginModule::InitState()
{
    if (_StoveSDKLibraryHandle)
    {
        StovePCInitState state = StovePC_GetInitState();
        static auto prevState = STOVE_PC_INIT_NONE;

        if (prevState != state)
        {
            if (state == STOVE_PC_INIT_FAIL)
            {
                UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : InitState -->"));
                UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetInitState is error = %d"), state);
            }
            else
            {
                UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : InitState -->"));
                UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetInitState is result = %d"), state);
            }

            prevState = state;
        }

        return state;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : InitState -->"));
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_INIT_FAIL;
    }
}

StovePCResult FStoveSDKPluginModule::RunCallback()
{
    static bool logOnce = true;
    if (logOnce)
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : RunCallback -->"));
    }
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_RunCallback();
        if (result != STOVE_PC_NO_ERROR)
        {
            if (logOnce)
            {
                UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_RunCallback is error = %d"), result);
            }
            
        }
        else
        {
            if (logOnce)
            {
                UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_RunCallback is result = %d"), result);
            }
            
        }
        logOnce = false; 
        return result;
    }
    else
    {
        logOnce = false;
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
    
}

StovePCResult FStoveSDKPluginModule::GetToken()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetToken -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetToken();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetToken is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetToken is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetUser()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetUser -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result =  StovePC_GetUser();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetUser is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetUser is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetOwnership()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetOwnership -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetOwnership();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetOwnership is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetOwnership is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetStat(const char* statId)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetStat -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetStat(statId);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetStat is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetStat is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::SetStat(const char* statId, const int statValue)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : SetStat -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_SetStat(statId, statValue);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_SetStat is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_SetStat is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetAchievement(const char* achievementId)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetAchievement -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetAchievement(achievementId);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetAchievement is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetAchievement is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetAllAchievement()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetAllAchievement -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetAllAchievement();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetAllAchievement is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetAllAchievement is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetRank(const char* leaderboardId, const unsigned int pageIndex, const unsigned int pageSize, bool includeMyRank)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetRank -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetRank(leaderboardId, pageIndex, pageSize,includeMyRank);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetRank is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetRank is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::SetGameProfile(const char * worldId, const __int64 charactorNo)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : SetGameProfile -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_SetGameProfile(worldId, charactorNo);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_SetGameProfile is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_SetGameProfile is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetAutoPopup()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetAutoPopup -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetAutoPopup();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetAutoPopup is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetAutoPopup is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetManualPopup(const char * resourceKey)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetManualPopup -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetManualPopup(resourceKey);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetManualPopup is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetManualPopup is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetNewsPopup()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetNewsPopup -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetNewsPopup();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetNewsPopup is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetNewsPopup is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetCouponPopup()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetCouponPopup -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetCouponPopup();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetCouponPopup is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetCouponPopup is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::GetCommunityPopup()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetCommunityPopup -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_GetCommunityPopup();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_GetCommunityPopup is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_GetCommunityPopup is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

FString FStoveSDKPluginModule::GetSDKVersion()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetSDKVersion -->"));
    FString version = StovePC_GetSDKVersion();

    UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePCSDK Version : %s"), *version);

    return version;
}


StovePCResult FStoveSDKPluginModule::SetLanguage(const char* language)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : SetLanguage -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_SetLanguage(language);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_SetLanguage is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_SetLanguage is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}
 
 StovePCResult FStoveSDKPluginModule::SetPopupDisallowed(const int popupId, const int days)
 {
     UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : SetPopupDisallowed -->"));
     if (_StoveSDKLibraryHandle)
     {
         StovePCResult result = StovePC_SetPopupDisallowed(popupId, days);
         if (result != STOVE_PC_NO_ERROR)
         {
             UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_SetPopupDisallowed is error = %d"), result);
         }
         else
         {
             UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_SetPopupDisallowed is result = %d"), result);
         }
         return result;
     }
     else
     {
         UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
         return STOVE_PC_MANAGED_EXCEPTION;
     }
 }

 wchar_t* FStoveSDKPluginModule::TranslateLanguage(const char* stringId)
 {
     UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : TranslateLanguage -->"));
     if (_StoveSDKLibraryHandle)
     {
         return StovePC_TranslateLanguage(stringId);
     }
     else
     {
         UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
         return L"plugin stove handle is null";
     }
 }

 StovePCTraceHint FStoveSDKPluginModule::GetTraceHint()
 {
     UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : GetTraceHint -->"));
     if (_StoveSDKLibraryHandle)
     {
         return StovePC_GetTraceHint();
     }
     else
     {
         UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
         return StovePCTraceHint{STOVE_PC_MANAGED_EXCEPTION};
     }
 }

StovePCResult FStoveSDKPluginModule::StashCustomEvent(const char* name, const char* category1, const char* category2, float simpleValue, const StovePCCustomEventParameter* params, int paramsSize)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : StashCustomEvent -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_StashCustomEvent(name, category1, category2, simpleValue, params, paramsSize);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_StashCustomEvent is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_StashCustomEvent is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::FetchShopCategories()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : FetchShopCategories -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_FetchShopCategories();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_FetchShopCategories is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_FetchShopCategories is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::FetchProducts(const char* categoryId, const bool isRefresh)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : FetchProducts -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_FetchProducts(categoryId, isRefresh);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_FetchProducts is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_FetchProducts is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::StartPurchase(const StovePCOrderProduct* products, int productsSize)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : StartPurchase -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_StartPurchase(products, productsSize);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_StartPurchase is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_StartPurchase is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::ConfirmPurchase(const __int64 transactionMasterNo)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : ConfirmPurchase -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_ConfirmPurchase(transactionMasterNo);
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_ConfirmPurchase is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_ConfirmPurchase is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCPurchaseVerification FStoveSDKPluginModule::VerifyPurchase(const char* categoryId, const __int64 productId)
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : VerifyPurchase -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCPurchaseVerification verify = StovePC_VerifyPurchase(categoryId, productId);
        if (verify.result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_VerifyPurchase is error = %d"), verify.result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_VerifyPurchase is result = %d"), verify.result);
        }
        return verify;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return StovePCPurchaseVerification{ STOVE_PC_MANAGED_EXCEPTION, false };
    }
}

StovePCResult FStoveSDKPluginModule::FetchInventory()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : FetchInventory -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_FetchInventory();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_FetchInventory is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_FetchInventory is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

StovePCResult FStoveSDKPluginModule::UnInit()
{
    UE_LOG(LogStoveSDKPlugin, Log, TEXT("PluginModule : UnInit -->"));
    if (_StoveSDKLibraryHandle)
    {
        StovePCResult result = StovePC_UnInit();
        if (result != STOVE_PC_NO_ERROR)
        {
            UE_LOG(LogStoveSDKPlugin, Error, TEXT("StovePC_UnInit is error = %d"), result);
        }
        else
        {
            UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_UnInit is result = %d"), result);
        }
        return result;
    }
    else
    {
        UE_LOG(LogStoveSDKPlugin, Error, TEXT("PluginModule error : handle is null"));
        return STOVE_PC_MANAGED_EXCEPTION;
    }
}

#undef LOCTEXT_NAMESPACE

IMPLEMENT_MODULE(FStoveSDKPluginModule, StoveSDKPlugin)

DEFINE_LOG_CATEGORY(LogStoveSDKPlugin);
